from design.plone.policy.patches.collective_volto_formsupport import (
    patch_FormDataExportGet_get_data,
)
from design.plone.policy.patches.collective_volto_formsupport import (
    patch_FormDataStore_methods,
)
from design.plone.policy.patches.collective_volto_formsupport import (
    patch_SubmitPost_reply,
)


def apply():
    patch_FormDataExportGet_get_data()
    patch_SubmitPost_reply()
    patch_FormDataStore_methods()



# -*- coding: utf-8 -*-
import collective.feedback
import collective.MockMailHost
import collective.taxonomy
import collective.volto.dropdownmenu
import collective.volto.formsupport
import collective.volto.secondarymenu
import collective.volto.slimheader
import collective.volto.socialsettings
import collective.volto.subfooter
import collective.volto.subsites
import collective.z3cform.datagridfield
import design.plone.contenttypes
import design.plone.policy
import plone.app.contentlisting
import redturtle.faq
import redturtle.voltoplugin.editablefooter
import souper.plone
from design.plone.contenttypes.testing import (
    DesignPloneContenttypesLayer,
    DesignPloneContenttypesRestApiLayer,
)
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import FunctionalTesting, IntegrationTesting, applyProfile
from plone.testing import z2
from zope.globalrequest import setRequest


class FauxRequest(dict):
    URL = "http://nohost"


class DesignPlonePolicyLayer(DesignPloneContenttypesLayer):
    def setUpZope(self, app, configurationContext):
        # Load any other ZCML that is required for your tests.
        # The z3c.autoinclude feature is disabled in the Plone fixture base
        # layer.

        super().setUpZope(app, configurationContext)
        self.loadZCML(package=design.plone.policy)
        self.loadZCML(package=collective.feedback)
        self.loadZCML(package=collective.volto.formsupport)
        self.loadZCML(package=collective.volto.dropdownmenu)
        self.loadZCML(package=collective.volto.secondarymenu)
        self.loadZCML(package=collective.volto.socialsettings)
        self.loadZCML(package=collective.volto.subsites)
        self.loadZCML(package=redturtle.voltoplugin.editablefooter)
        self.loadZCML(package=collective.volto.subfooter)
        self.loadZCML(package=collective.volto.slimheader)
        self.loadZCML(package=collective.taxonomy)
        self.loadZCML(name="overrides.zcml", package=design.plone.contenttypes)
        self.loadZCML(package=plone.app.contentlisting)
        self.loadZCML(package=souper.plone)
        self.loadZCML(package=redturtle.faq)

    def setUpPloneSite(self, portal):
        super().setUpPloneSite(portal)
        request = portal.REQUEST
        setRequest(request)
        applyProfile(portal, "design.plone.policy:default")


DESIGN_PLONE_POLICY_FIXTURE = DesignPlonePolicyLayer()


DESIGN_PLONE_POLICY_INTEGRATION_TESTING = IntegrationTesting(
    bases=(DESIGN_PLONE_POLICY_FIXTURE,),
    name="DesignPlonePolicyLayer:IntegrationTesting",
)


DESIGN_PLONE_POLICY_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(DESIGN_PLONE_POLICY_FIXTURE,),
    name="DesignPlonePolicyLayer:FunctionalTesting",
)


DESIGN_PLONE_POLICY_ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(
        DESIGN_PLONE_POLICY_FIXTURE,
        REMOTE_LIBRARY_BUNDLE_FIXTURE,
        z2.ZSERVER_FIXTURE,
    ),
    name="DesignPlonePolicyLayer:AcceptanceTesting",
)


class DesignPlonePolicyLimitRootAddablesLayer(DesignPlonePolicyLayer):
    def setUpZope(self, app, configurationContext):
        # Load any other ZCML that is required for your tests.
        # The z3c.autoinclude feature is disabled in the Plone fixture base
        # layer.

        super().setUpZope(app, configurationContext)
        self.loadZCML(package=design.plone.policy)

    def setUpPloneSite(self, portal):
        super().setUpPloneSite(portal)
        request = portal.REQUEST
        setRequest(request)
        import pdb

        pdb.set_trace()
        applyProfile(portal, "design.plone.policy.limit_root_addables:default")


DESIGN_PLONE_POLICY_LIMIT_ROOT_ADDABLES_INTEGRATION_TESTING = IntegrationTesting(
    bases=(DESIGN_PLONE_POLICY_INTEGRATION_TESTING,),
    name="DesignPlonePolicyLimitRootAddablesLayer:IntegrationTesting",
)


class DesignPlonePolicyRestApiLayer(DesignPloneContenttypesRestApiLayer):
    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        super().setUpZope(app, configurationContext)

        self.loadZCML(package=collective.z3cform.datagridfield)
        self.loadZCML(package=design.plone.policy)
        self.loadZCML(package=collective.feedback)
        self.loadZCML(package=collective.volto.formsupport)
        self.loadZCML(package=collective.volto.dropdownmenu)
        self.loadZCML(package=collective.volto.secondarymenu)
        self.loadZCML(package=collective.volto.socialsettings)
        self.loadZCML(package=collective.volto.subsites)
        self.loadZCML(package=redturtle.voltoplugin.editablefooter)
        self.loadZCML(package=collective.volto.subfooter)
        self.loadZCML(package=collective.volto.slimheader)
        self.loadZCML(package=redturtle.volto)
        self.loadZCML(package=collective.taxonomy)
        self.loadZCML(name="overrides.zcml", package=design.plone.contenttypes)
        self.loadZCML(package=design.plone.policy)
        self.loadZCML(package=plone.app.contentlisting)
        self.loadZCML(package=redturtle.faq)
        self.loadZCML(package=souper.plone)
        self.loadZCML(package=redturtle.faq)

    def setUpPloneSite(self, portal):
        super().setUpPloneSite(portal)
        request = portal.REQUEST
        setRequest(request)
        applyProfile(portal, "design.plone.policy:default")


DESIGN_PLONE_POLICY_API_FIXTURE = DesignPlonePolicyRestApiLayer()
DESIGN_PLONE_POLICY_API_INTEGRATION_TESTING = IntegrationTesting(
    bases=(DESIGN_PLONE_POLICY_API_FIXTURE,),
    name="DesignPlonePolicyRestApiLayer:Integration",
)

DESIGN_PLONE_POLICY_API_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(DESIGN_PLONE_POLICY_API_FIXTURE, z2.ZSERVER_FIXTURE),
    name="DesignPlonePolicyRestApiLayer:Functional",
)



# -*- coding: utf-8 -*-
# flake8: noqa
STRUTTURA = [
    {
        "id": "amministrazione-trasparente",
        "title": "Amministrazione Trasparente",
        "type": "Document",
        "children": [
            {
                "id": " Disposizioni generali",
                "title": "Disposizioni generali",
                "type": "Document",
                "children": [
                    {
                        "id": "Piano triennale per la prevenzione della corruzione e della trasparenza",
                        "title": "Piano triennale per la prevenzione della corruzione e della trasparenza",
                        "type": "Document",
                    },
                    {
                        "id": "Atti generali",
                        "title": "Atti generali",
                        "type": "Document",
                    },
                    {
                        "id": "Oneri informativi per cittadini e imprese",
                        "title": "Oneri informativi per cittadini e imprese",
                        "type": "Document",
                    },
                    {
                        "id": "Burocrazia zero",
                        "title": "Burocrazia zero",
                        "type": "Document",
                    },
                ],
            },
            {
                "id": "Organizzazione",
                "title": "Organizzazione",
                "type": "Document",
                "children": [
                    {
                        "id": "Titolari di incarichi politici, di amministrazione, di direzione o di governo",
                        "title": "Titolari di incarichi politici, di amministrazione, di direzione o di governo",
                        "type": "Document",
                    },
                    {
                        "id": "Sanzioni per mancata comunicazione dei dati ",
                        "title": "Sanzioni per mancata comunicazione dei dati",
                        "type": "Document",
                    },
                    {
                        "id": "Rendiconti gruppi consiliari regionali/provinciali",
                        "title": "Rendiconti gruppi consiliari regionali/provinciali",
                        "type": "Document",
                    },
                    {
                        "id": "Articolazione degli uffici",
                        "title": "Articolazione degli uffici",
                        "type": "Document",
                    },
                    {
                        "id": "Telefono e posta elettronica",
                        "title": "Telefono e posta elettronica",
                        "type": "Document",
                    },
                ],
            },
            {
                "id": "Consulenti e collaboratori",
                "title": "Consulenti e collaboratori",
                "type": "Document",
                "children": [
                    {
                        "id": "Titolari di incarichi di collaborazione o consulenza",
                        "title": "Consulenti e collaboratori",
                        "type": "Document",
                    },
                ],
            },
            {
                "id": "Personale",
                "title": "Personale",
                "type": "Document",
                "children": [
                    {
                        "id": "Titolari di incarichi dirigenziali amministrativi di vertice",
                        "title": "Titolari di incarichi dirigenziali amministrativi di vertice",
                        "type": "Document",
                    },
                    {
                        "id": "Titolari di incarichi dirigenziali (dirigenti non generali)",
                        "title": "Titolari di incarichi dirigenziali (dirigenti non generali)",
                        "type": "Document",
                    },
                    {
                        "id": "Dirigenti cessati",
                        "title": "Dirigenti cessati",
                        "type": "Document",
                    },
                    {
                        "id": "Sanzioni per mancata comunicazione dei dati",
                        "title": "Sanzioni per mancata comunicazione dei dati",
                        "type": "Document",
                    },
                    {
                        "id": "Posizioni organizzative",
                        "title": "Posizioni organizzative",
                        "type": "Document",
                    },
                    {
                        "id": "Dotazione organica",
                        "title": "Dotazione organica",
                        "type": "Document",
                    },
                    {
                        "id": "Personale non a tempo indeterminato",
                        "title": "Personale non a tempo indeterminato",
                        "type": "Document",
                    },
                    {
                        "id": "Tassi di assenza",
                        "title": "Tassi di assenza",
                        "type": "Document",
                    },
                    {
                        "id": "Incarichi conferiti e autorizzati ai dipendenti (dirigenti e non dirigenti)",
                        "title": "Incarichi conferiti e autorizzati ai dipendenti (dirigenti e non dirigenti)",
                        "type": "Document",
                    },
                    {
                        "id": "Contrattazione collettiva",
                        "title": "Contrattazione collettiva",
                        "type": "Document",
                    },
                    {
                        "id": "Contrattazione integrativa",
                        "title": "Contrattazione integrativa",
                        "type": "Document",
                    },
                    {
                        "id": "OIV",
                        "title": "OIV",
                        "type": "Document",
                    },
                ],
            },
            {
                "id": "Bandi di concorso",
                "title": "Bandi di concorso",
                "type": "Document",
            },
            {
                "id": "Performance",
                "title": "Performance",
                "type": "Document",
                "children": [
                    {
                        "id": "Sistema di misurazione e valutazione della Performance",
                        "title": "Sistema di misurazione e valutazione della Performance",
                        "type": "Document",
                    },
                    {
                        "id": "Piano della Performance",
                        "title": "Piano della Performance",
                        "type": "Document",
                    },
                    {
                        "id": "Relazione sulla Performance",
                        "title": "Relazione sulla Performance",
                        "type": "Document",
                    },
                    {
                        "id": "Ammontare complessivo dei premi",
                        "title": "Ammontare complessivo dei premi",
                        "type": "Document",
                    },
                    {
                        "id": "Dati relativi ai premi",
                        "title": "Dati relativi ai premi",
                        "type": "Document",
                    },
                    {
                        "id": "Benessere organizzativo",
                        "title": "Benessere organizzativo",
                        "type": "Document",
                    },
                ],
            },
            {
                "id": "Enti controllati",
                "title": "Enti controllati",
                "type": "Document",
                "children": [
                    {
                        "id": "Enti pubblici vigilati",
                        "title": "Enti pubblici vigilati",
                        "type": "Document",
                    },
                    {
                        "id": "Società partecipate",
                        "title": "Società partecipate",
                        "type": "Document",
                    },
                    {
                        "id": "Enti di diritto privato controllati",
                        "title": "Enti di diritto privato controllati",
                        "type": "Document",
                    },
                    {
                        "id": "Rappresentazione grafica",
                        "title": "Rappresentazione grafica",
                        "type": "Document",
                    },
                ],
            },
            {
                "id": "Attività e procedimenti",
                "title": "Attività e procedimenti",
                "type": "Document",
                "children": [
                    {
                        "id": "Tipologie di procedimento",
                        "title": "Tipologie di procedimento",
                        "type": "Document",
                    },
                    {
                        "id": "Monitoraggio tempi procedimentali",
                        "title": "Monitoraggio tempi procedimentali",
                        "type": "Document",
                    },
                    {
                        "id": "Dichiarazioni sostitutive e acquisizione d'ufficio dei dati",
                        "title": "Dichiarazioni sostitutive e acquisizione d'ufficio dei dati",
                        "type": "Document",
                    },
                ],
            },
            {
                "id": "Provvedimenti",
                "title": "Provvedimenti",
                "type": "Document",
                "children": [
                    {
                        "id": "Provvedimenti organi indirizzo politico",
                        "title": "Provvedimenti organi indirizzo politico",
                        "type": "Document",
                    },
                    {
                        "id": "Provvedimenti organi indirizzo politico",
                        "title": "Provvedimenti organi indirizzo politico",
                        "type": "Document",
                    },
                    {
                        "id": "Provvedimenti dirigenti amministrativi",
                        "title": "Provvedimenti dirigenti amministrativi",
                        "type": "Document",
                    },
                    {
                        "id": "Provvedimenti dirigenti amministrativi",
                        "title": "Provvedimenti dirigenti amministrativi",
                        "type": "Document",
                    },
                ],
            },
            {
                "id": "Controlli sulle imprese",
                "title": "Controlli sulle imprese",
                "type": "Document",
            },
            {
                "id": "Bandi di gara e contratti",
                "title": "Bandi di gara e contratti",
                "type": "Document",
                "children": [
                    {
                        "id": "Informazioni sulle singole procedure in formato tabellare",
                        "title": "Informazioni sulle singole procedure in formato tabellare",
                        "type": "Document",
                    },
                    {
                        "id": "Atti delle amministrazioni aggiudicatrici e degli enti aggiudicatori distintamente per ogni procedura",
                        "title": "Atti delle amministrazioni aggiudicatrici e degli enti aggiudicatori distintamente per ogni procedura",
                        "type": "Document",
                    },
                ],
            },
            {
                "id": "Sovvenzioni,contributi,sussidi,vantaggi economici",
                "title": "Sovvenzioni, contributi, sussidi, vantaggi economici",
                "type": "Document",
                "children": [
                    {
                        "id": "Criteri e modalità",
                        "title": "Criteri e modalità",
                        "type": "Document",
                    },
                    {
                        "id": "Atti di concessione",
                        "title": "Atti di concessione",
                        "type": "Document",
                    },
                ],
            },
            {
                "id": "Bilanci",
                "title": "Bilanci",
                "type": "Document",
                "children": [
                    {
                        "id": "Bilancio preventivo e consuntivo",
                        "title": "Bilancio preventivo e consuntivo",
                        "type": "Document",
                    },
                    {
                        "id": "Piano degli indicatori e risultati attesi di bilancio",
                        "title": "Piano degli indicatori e risultati attesi di bilancio",
                        "type": "Document",
                    },
                ],
            },
            {
                "id": "Beni immobili e gestione patrimonio",
                "title": "Beni immobili e gestione patrimonio",
                "type": "Document",
                "children": [
                    {
                        "id": "Patrimonio immobiliare",
                        "title": "Patrimonio immobiliare",
                        "type": "Document",
                    },
                    {
                        "id": "Canoni di locazione o affitto",
                        "title": "Canoni di locazione o affitto",
                        "type": "Document",
                    },
                ],
            },
            {
                "id": "Controlli e rilievi sull'amministrazione",
                "title": "Controlli e rilievi sull'amministrazione",
                "type": "Document",
                "children": [
                    {
                        "id": "Organismi indipendenti di valutazione, nuclei di valutazione o altri organismi con funzioni analoghe",
                        "title": "Organismi indipendenti di valutazione, nuclei di valutazione o altri organismi con funzioni analoghe",
                        "type": "Document",
                    },
                    {
                        "id": "Organi di revisione amministrativa e contabile",
                        "title": "Organi di revisione amministrativa e contabile",
                        "type": "Document",
                    },
                    {
                        "id": "Corte dei conti",
                        "title": "Corte dei conti",
                        "type": "Document",
                    },
                ],
            },
            {
                "id": "Servizi erogati",
                "title": "Servizi erogati",
                "type": "Document",
                "children": [
                    {
                        "id": "Carta dei servizi e standard di qualità",
                        "title": "Carta dei servizi e standard di qualità",
                        "type": "Document",
                    },
                    {
                        "id": "Class Action",
                        "title": "Class Action",
                        "type": "Document",
                    },
                    {
                        "id": "Costi contabilizzati",
                        "title": "Costi contabilizzati",
                        "type": "Document",
                    },
                    {
                        "id": "Liste di attesa",
                        "title": "Liste di attesa",
                        "type": "Document",
                    },
                    {
                        "id": "Servizi in rete",
                        "title": "Servizi in rete",
                        "type": "Document",
                    },
                ],
            },
            {
                "id": "Pagamenti dell'amministrazione",
                "title": "Pagamenti dell'amministrazione",
                "type": "Document",
                "children": [
                    {
                        "id": "Dati sui pagamenti",
                        "title": "Dati sui pagamenti",
                        "type": "Document",
                    },
                    {
                        "id": "Dati sui pagamenti del servizio sanitario nazionale",
                        "title": "Dati sui pagamenti del servizio sanitario nazionale",
                        "type": "Document",
                    },
                    {
                        "id": "Indicatore di tempestività dei pagamenti",
                        "title": "Indicatore di tempestività dei pagamenti",
                        "type": "Document",
                    },
                    {
                        "id": "IBAN e pagamenti informatici",
                        "title": "IBAN e pagamenti informatici",
                        "type": "Document",
                    },
                ],
            },
            {
                "id": "Opere pubbliche",
                "title": "Opere pubbliche",
                "type": "Document",
                "children": [
                    {
                        "id": "Nuclei di valutazione e verifica degli investimenti pubblici",
                        "title": "Nuclei di valutazione e verifica degli investimenti pubblici",
                        "type": "Document",
                    },
                    {
                        "id": "Atti di programmazione delle opere pubbliche",
                        "title": "Atti di programmazione delle opere pubbliche",
                        "type": "Document",
                    },
                    {
                        "id": "Tempi costi e indicatori di realizzazione delle opere pubbliche",
                        "title": "Tempi costi e indicatori di realizzazione delle opere pubbliche",
                        "type": "Document",
                    },
                ],
            },
            {
                "id": "Pianificazione e governo del territorio",
                "title": "Pianificazione e governo del territorio",
                "type": "Document",
            },
            {
                "id": "Informazioni ambientali",
                "title": "Informazioni ambientali",
                "type": "Document",
            },
            {
                "id": "Strutture sanitarie private accreditate",
                "title": "Strutture sanitarie private accreditate",
                "type": "Document",
            },
            {
                "id": "Interventi straordinari e di emergenza",
                "title": "Interventi straordinari e di emergenza",
                "type": "Document",
            },
            {
                "id": "Altri contenuti - Prevenzione della corruzione",
                "title": "Prevenzione della corruzione",
                "type": "Document",
            },
            {
                "id": "Altri contenuti - Accesso Civico",
                "title": "Accesso Civico",
                "type": "Document",
            },
            {
                "id": "Altri contenuti - Accessibilità e catalogo di dati, metadati e banche dati",
                "title": "Accessibilità e catalogo di dati, metadati e banche dati",
                "type": "Document",
            },
            {
                "id": "Altri Contenuti",
                "title": "Altri contenuti",
                "type": "Document",
            },
        ],
    },
]



# -*- coding: utf-8 -*-
from design.plone.policy.testing import DESIGN_PLONE_POLICY_INTEGRATION_TESTING
from design.plone.policy.utils import TASSONOMIA_AMMINISTRAZIONE
from design.plone.policy.utils import TASSONOMIA_ARGOMENTI
from design.plone.policy.utils import TASSONOMIA_NEWS
from design.plone.policy.utils import TASSONOMIA_ORGANI_GOVERNO
from design.plone.policy.utils import TASSONOMIA_SERVIZI
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.i18n.normalizer.interfaces import IURLNormalizer
from plone.restapi.behaviors import IBlocks
from zope.component import getUtility

import unittest


# TODO: rework tests
class TestInitialStructureCreation(unittest.TestCase):
    layer = DESIGN_PLONE_POLICY_INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]

        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def normalize_ids(self, string):
        return getUtility(IURLNormalizer).normalize(string)

    def check_initial_blocks(self, obj):
        self.assertTrue(IBlocks.providedBy(obj))
        self.assertEqual(len(obj.blocks.values()), 1)
        self.assertEqual(len(obj.blocks_layout["items"]), 1)

    def check_children_initial_blocks(self, obj):
        for child in obj.listFolderContents():
            self.check_initial_blocks(child)

    def test_first_level_created(self):
        no_blocks = [
            "prenotazione-appuntamento",
            "segnalazione-disservizio",
            "richiesta-di-assistenza",
            "amministrazione-trasparente",
            "informativa-privacy",
            "note-legali",
            "media-policy",
        ]
        for child in self.portal.listFolderContents():
            if child.id == "leggi-le-faq":
                self.assertEqual(child.portal_type, "FaqFolder")
            elif child.id == "dichiarazione-di-accessibilita":
                self.assertEqual(child.portal_type, "Link")
            else:
                self.assertEqual(child.portal_type, "Document")
                if child.id not in no_blocks:
                    self.assertEqual(len(child.blocks.values()), 1)
                    self.assertEqual(len(child.blocks_layout["items"]), 1)

    def test_amministrazione_section(self):
        amministrazione = self.portal["amministrazione"]
        self.assertEqual(
            amministrazione.keys(),
            [self.normalize_ids(x) for x in TASSONOMIA_AMMINISTRAZIONE],
        )
        for x in TASSONOMIA_AMMINISTRAZIONE:
            self.assertEqual(
                amministrazione.get(self.normalize_ids(x), None).portal_type, "Document"
            )
        self.check_children_initial_blocks(amministrazione)

        self.assertEqual(
            amministrazione["organi-di-governo"].keys(),
            [self.normalize_ids(x) for x in TASSONOMIA_ORGANI_GOVERNO],
        )
        for child in amministrazione["organi-di-governo"].listFolderContents():
            self.assertEqual(child.portal_type, "Document")
            self.check_initial_blocks(child)

    def test_servizi_section(self):
        servizi = self.portal["servizi"]
        self.assertEqual(
            servizi.keys(),
            [self.normalize_ids(x) for x in TASSONOMIA_SERVIZI],
        )

        for child in servizi.listFolderContents():
            self.assertEqual(child.portal_type, "Document")
            self.check_initial_blocks(child)

    def test_novita_section(self):
        folder = self.portal["novita"]
        self.assertEqual(
            folder.keys(), [self.normalize_ids(x) for x in TASSONOMIA_NEWS]
        )

        for child in folder.listFolderContents():
            self.assertEqual(child.portal_type, "Document")
            self.check_initial_blocks(child)

    def test_argomenti_section(self):
        folder = self.portal["argomenti"]
        self.assertEqual(folder.portal_type, "Document")
        self.assertEqual(
            folder.keys(), [self.normalize_ids(x) for x in TASSONOMIA_ARGOMENTI]
        )
        self.assertEqual(folder.portal_type, "Document")
        for child in folder.listFolderContents():
            self.assertEqual(child.portal_type, "Pagina Argomento")
            self.assertEqual(len(child.blocks.values()), 1)
            self.assertEqual(len(child.blocks_layout["items"]), 1)

    def test_enabled_blocks_contents_have_defaults(self):
        brains = self.portal.portal_catalog(
            object_provides="plone.restapi.behaviors.IBlocks"
        )
        for brain in brains:
            self.check_initial_blocks(brain.getObject())



# -*- coding: utf-8 -*-
from collective.volto.dropdownmenu.interfaces import IDropDownMenu
from collective.volto.secondarymenu.interfaces import ISecondaryMenu
from collective.volto.subfooter.interfaces import ISubfooter
from plone import api
from plone.i18n.normalizer.interfaces import IIDNormalizer
from plone.restapi.behaviors import IBlocks
from plone.restapi.interfaces import ISerializeToJsonSummary
from Products.CMFPlone.interfaces import ISelectableConstrainTypes
from redturtle.voltoplugin.editablefooter.interfaces import IEditableFooterSettings
from uuid import uuid4
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.globalrequest import getRequest

import json
import time


TASSONOMIA_PRIMO_LIVELLO = [
    "Amministrazione",
    "Novità",
    "Servizi",
    "Vivere il Comune",
    "Argomenti",
]

TASSONOMIA_SERVIZI = [
    "Educazione e formazione",
    "Salute, benessere e assistenza",
    "Vita lavorativa",
    "Mobilità e trasporti",
    "Catasto e urbanistica",
    "Anagrafe e stato civile",
    "Turismo",
    "Giustizia e sicurezza pubblica",
    "Tributi, finanze e contravvenzioni",
    "Cultura e tempo libero",
    "Ambiente",
    "Imprese e commercio",
    "Autorizzazioni",
    "Appalti pubblici",
    "Agricoltura e pesca",
]

TASSONOMIA_DOCUMENTI = [
    "Documenti albo pretorio",
    "Modulistica",
    "Documenti funzionamento interno",
    "Atti normativi",
    "Accordi tra enti",
    "Documenti attività politica",
    "Documenti (tecnici) di supporto",
    "Istanze",
    "Documenti di programmazione e rendicontazione",
    "Dataset",
]

TASSONOMIA_NEWS = ["Notizie", "Comunicati", "Avvisi"]
TASSONOMIA_AMMINISTRAZIONE = [
    "Organi di governo",
    "Aree amministrative",
    "Uffici",
    "Enti e fondazioni",
    "Politici",
    "Personale Amministrativo",
    "Documenti e dati",
]

TASSONOMIA_VIVERE_IL_COMUNE = ["Luoghi", "Eventi"]

TASSONOMIA_ORGANI_GOVERNO = [
    "Giunta comunale",
    "Consiglio comunale",
    "Commissione",
]

TASSONOMIA_ARGOMENTI = [
    "Accesso all'informazione",
    "Acqua",
    "Agricoltura",
    "Animale domestico",
    "Aria",
    "Assistenza agli invalidi",
    "Assistenza sociale",
    "Associazioni",
    "Bilancio",
    "Commercio all'ingrosso",
    "Commercio al minuto",
    "Commercio ambulante",
    "Comunicazione istituzionale",
    "Comunicazione politica",
    "Concorsi",
    "Covid-19",
    "Elezioni",
    "Energie rinnovabili",
    "Estero",
    "Foreste",
    "Formazione professionale",
    "Gemellaggi",
    "Gestione rifiuti",
    "Giustizia",
    "Igiene pubblica",
    "Immigrazione",
    "Imposte",
    "Imprese",
    "Inquinamento",
    "Integrazione sociale",
    "Isolamento termico",
    "Istruzione",
    "Lavoro",
    "Matrimonio",
    "Mercato",
    "Mobilità sostenibile",
    "Morte",
    "Nascita",
    "Parcheggi",
    "Patrimonio culturale",
    "Pesca",
    "Piano di sviluppo",
    "Pista ciclabile",
    "Politica commerciale",
    "Polizia",
    "Prodotti alimentari",
    "Protezione civile",
    "Residenza",
    "Risposta alle emergenze",
    "Sistema giuridico",
    "Spazio Verde",
    "Sport",
    "Sviluppo sostenibile",
    "Tassa sui servizi",
    "Tempo libero",
    "Trasparenza amministrativa",
    "Trasporto pubblico",
    "Turismo",
    "Urbanizzazione",
    "Viaggi",
    "Zone pedonali",
    "ZTL",
]

TASSONOMIA_FOOTER = [
    {"title": "Leggi le FAQ", "type": "FaqFolder", "data-element": "faq"},
    {
        "title": "Prenotazione appuntamento",
        "type": "Document",
        "data-element": "appointment-booking",
    },
    {
        "title": "Segnalazione disservizio",
        "type": "Document",
        "data-element": "report-inefficiency",
    },
    {"title": "Richiesta di assistenza", "type": "Document"},
    {"title": "Amministrazione trasparente", "type": "Document"},
    {
        "title": "Informativa privacy",
        "type": "Document",
        "data-element": "privacy-policy-link",
    },
    {"title": "Note legali", "type": "Document"},
    {
        "title": "Dichiarazione di accessibilità",
        "type": "Link",
        "data-element": "accessibility-link",
    },
]


def folderSubstructureGenerator(title, types=[]):
    container = api.portal.get()

    normalizer = getUtility(IIDNormalizer)
    safe_id = normalizer.normalize(title)

    root_path = "/".join(container.getPhysicalPath())
    path = root_path + "/" + safe_id

    if api.content.get(path=path):
        return

    tree_root = api.content.create(container=container, type="Document", title=title)
    create_default_blocks(context=tree_root)

    api.content.transition(obj=tree_root, transition="publish")
    if types:
        restrict_types(context=tree_root, types=types)

    if title == "Servizi":
        for ts in TASSONOMIA_SERVIZI:
            child = api.content.create(container=tree_root, type="Document", title=ts)
            create_default_blocks(context=child)
    elif title == "Vivere il Comune":
        for td in TASSONOMIA_VIVERE_IL_COMUNE:
            child = api.content.create(container=tree_root, type="Document", title=td)
            create_default_blocks(context=child)
    elif title == "Novità":
        for tn in TASSONOMIA_NEWS:
            child = api.content.create(container=tree_root, type="Document", title=tn)
            create_default_blocks(context=child)

    elif title == "Amministrazione":
        for ta in TASSONOMIA_AMMINISTRAZIONE:
            child = api.content.create(container=tree_root, type="Document", title=ta)
            create_default_blocks(context=child)
            if ta == "Documenti e dati":
                for tdd in TASSONOMIA_DOCUMENTI:
                    grandchild = api.content.create(
                        container=child, type="Document", title=tdd
                    )
                    create_default_blocks(context=grandchild)
            elif ta == "Organi di governo":
                for tog in TASSONOMIA_ORGANI_GOVERNO:
                    grandchild = api.content.create(
                        container=child, type="Document", title=tog
                    )
                    create_default_blocks(context=grandchild)

    elif title == "Argomenti":
        for ta in TASSONOMIA_ARGOMENTI:
            child = api.content.create(
                container=tree_root, type="Pagina Argomento", title=ta
            )
            create_default_blocks(context=child)


def restrict_types(context, types):
    constraints = ISelectableConstrainTypes(context)
    constraints.setConstrainTypesMode(1)
    constraints.setLocallyAllowedTypes(types)


def create_default_blocks(context):
    if IBlocks.providedBy(context):
        title_uuid = str(uuid4())
        context.blocks = {title_uuid: {"@type": "title"}}
        context.blocks_layout = {"items": [title_uuid]}


def create_footer():
    container = api.portal.get()
    # Generate needed pages
    for item in TASSONOMIA_FOOTER:
        obj = api.content.create(
            container=container,
            type=item.get("type", "Pagina"),
            title=item.get("title", "Titolo"),
        )
        create_default_blocks(context=obj)
        api.content.transition(obj=obj, transition="publish")
        item["path"] = f"/{obj.getId()}"
        obj.exclude_from_nav = True
        if obj.portal_type == "FaqFolder":
            setattr(obj, "icon", None)
            obj.reindexObject()
        if obj.portal_type == "Link":
            setattr(obj, "remoteUrl", "")

    # generate the payload for settings
    payload_items = [
        {
            "id": str(int(time.time() * 1000)) + "1",
            "newsletterSubscribe": False,
            "showSocial": False,
            "text": {"data": "<p><br/></p>"},
            "title": "Contatti",
            "titleLink": [],
            "visible": True,
        }
    ]
    markup = ""
    for item in TASSONOMIA_FOOTER[: len(TASSONOMIA_FOOTER) // 2]:
        if "data-element" in item:
            markup += '<li><a data-element="{}" href="{}">{}</a></li>'.format(
                item.get("data-element"), item.get("path"), item.get("title")
            )
        else:
            markup += '<li><a href="{}">{}</a></li>'.format(
                item.get("path"), item.get("title")
            )
    text = '<ul keys="696me,24qcs,6j5h0,f4p0j">{}</ul>'.format(markup)
    payload_items.append(
        {
            "id": str(int(time.time() * 1000)) + "2",
            "newsletterSubscribe": False,
            "showSocial": False,
            "text": {"data": text},
            "titleLink": [],
            "visible": True,
        }
    )

    markup = ""
    for item in TASSONOMIA_FOOTER[len(TASSONOMIA_FOOTER) // 2 :]:
        if "data-element" in item:
            markup += '<li><a data-element="{}" href="{}">{}</a></li>'.format(
                item.get("data-element"), item.get("path"), item.get("title")
            )
        else:
            markup += '<li><a href="{}">{}</a></li>'.format(
                item.get("path"), item.get("title")
            )
    text = '<ul keys="at8uf,f1h7r,d22s9,6lou">{}</ul>'.format(markup)
    payload_items.extend(
        [
            {
                "id": str(int(time.time() * 1000)) + "3",
                "newsletterSubscribe": False,
                "showSocial": False,
                "text": {"data": text},
                "titleLink": [],
                "visible": True,
            },
            {
                "id": str(int(time.time() * 1000)) + "4",
                "newsletterSubscribe": False,
                "showSocial": True,
                "text": {"data": "<p><br/></p>"},
                "title": "Seguici su",
                "titleLink": [],
                "visible": True,
            },
        ]
    )
    payload = [{"items": payload_items, "rootPath": "/"}]

    payload = json.dumps(payload)
    api.portal.set_registry_record(
        "footer_columns", payload, interface=IEditableFooterSettings
    )

    # generate content for subfooter
    # Generate needed pages
    obj = api.content.create(
        container=container,
        type="Document",
        title="Media Policy",
    )
    create_default_blocks(context=obj)
    obj.exclude_from_nav = True
    obj.reindexObject()

    payload = json.dumps(
        [
            {
                "rootPath": "/",
                "items": [
                    {
                        "title": "Media Policy",
                        "visible": True,
                        "href": f"/{obj.getId()}",
                    },
                    {"title": "Sitemap", "visible": True, "href": "/sitemap"},
                    {
                        "title": "Credits",
                        "visible": True,
                        "href": "https://www.io-comune.it/",
                    },
                ],
            }
        ]
    )
    api.portal.set_registry_record(
        "subfooter_configuration", payload, interface=ISubfooter
    )


def create_default_menu_item(context, obj):
    title_uuid = str(uuid4())
    title = obj.Title()
    uid = obj.UID()
    context["blocks_layout"] = {"items": [title_uuid]}
    context["blocks"] = {title_uuid: {"@type": "text"}}
    context["title"] = title
    context["mode"] = "simpleLink"
    context["visible"] = True
    context["linkUrl"] = [uid]
    if title == "Amministrazione":
        context["id_lighthouse"] = "management"
        context["showMoreText"] = "Vedi tutto"
        context["navigationRoot"] = [uid]
        context["showMoreLink"] = [uid]
    elif title == "Servizi":
        context["id_lighthouse"] = "all-services"
        context["navigationRoot"] = [uid]
        context["showMoreLink"] = [uid]
    elif title == "Novità":
        context["id_lighthouse"] = "news"
        context["navigationRoot"] = [uid]
        context["showMoreLink"] = [uid]
    elif title == "Vivere il Comune":
        context["id_lighthouse"] = "live"
    elif title == "Argomenti":
        context["id_lighthouse"] = "all-topics"
        del context["mode"]
        context["title"] = "Tutti gli argomenti..."

    return context


def create_menu():
    request = getRequest()
    amministrazione = api.content.get(path="/amministrazione")
    servizi = api.content.get(path="/servizi")
    novita = api.content.get(path="/novita")
    vivere = api.content.get(path="/vivere-il-comune")

    items = []
    for x in [amministrazione, novita, servizi, vivere]:
        json_serialized = getMultiAdapter(
            (x, request or x.REQUEST), ISerializeToJsonSummary
        )()
        items.append(create_default_menu_item(context=json_serialized, obj=x))

    mocked_payload = [
        {
            "rootPath": "/",
            "items": items,
        }
    ]
    payload = json.dumps(mocked_payload)
    api.portal.set_registry_record(
        "menu_configuration", payload, interface=IDropDownMenu
    )


def create_secondary_menu():
    request = getRequest()
    items = []
    argomenti = api.content.get(path="/argomenti")
    json_serialized = getMultiAdapter(
        (argomenti, request or argomenti.REQUEST), ISerializeToJsonSummary
    )()
    items.append(create_default_menu_item(context=json_serialized, obj=argomenti))
    mocked_payload = [{"rootPath": "/", "items": items}]
    payload = json.dumps(mocked_payload)
    api.portal.set_registry_record(
        "secondary_menu_configuration", payload, interface=ISecondaryMenu
    )



# -*- coding: utf-8 -*-
from design.plone.policy.testing import DESIGN_PLONE_POLICY_API_FUNCTIONAL_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.registry.interfaces import IRegistry
from plone.restapi.testing import RelativeSession
from Products.CMFPlone.interfaces import ISearchSchema
from transaction import commit
from zope.component import getUtility
from design.plone.contenttypes.controlpanels.settings import IDesignPloneSettings
import json
import unittest


class SearchFiltersAPITest(unittest.TestCase):
    layer = DESIGN_PLONE_POLICY_API_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        self.api_session_anon = RelativeSession(self.portal_url)
        self.api_session_anon.headers.update({"Accept": "application/json"})

        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def tearDown(self):
        self.api_session.close()
        self.api_session_anon.close()

    def test_endpoint_exists(self):
        response = self.api_session.get("/@search-filters")

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.headers.get("Content-Type"), "application/json")

    def test_endpoint_return_list_of_sections(self):
        response = self.api_session.get("/@search-filters").json()

        self.assertIn("sections", response)
        sections = response["sections"]

        self.assertEqual(len(sections), 1)
        self.assertEqual(len(sections[0]["items"]), 4)
        self.assertEqual("Amministrazione", sections[0]["items"][0]["title"])
        self.assertEqual("Servizi", sections[0]["items"][1]["title"])
        self.assertEqual("Novità", sections[0]["items"][2]["title"])
        self.assertEqual("Vivere il Comune", sections[0]["items"][3]["title"])

    def test_endpoint_return_list_of_topics_empty_if_no_topics(self):
        response = self.api_session.get("/@search-filters").json()

        self.assertIn("topics", response)
        self.assertEqual(len(response["topics"]), 62)

    def test_endpoint_return_list_of_topics_based_on_roles(self):
        api.content.create(
            container=self.portal, type="Pagina Argomento", title="private topic"
        )
        public_topic = api.content.create(
            container=self.portal, type="Pagina Argomento", title="public topic"
        )
        api.content.transition(obj=public_topic, transition="publish")
        commit()

        response = self.api_session.get("/@search-filters").json()
        self.assertIn("topics", response)
        self.assertEqual(len(response["topics"]), 64)

        response = self.api_session_anon.get("/@search-filters").json()

        self.assertIn("topics", response)
        self.assertEqual(len(response["topics"]), 1)

    def test_endpoint_return_list_of_searchable_types(self):
        response = self.api_session.get("/@search-filters").json()

        self.assertIn("portal_types", response)
        types = [x["id"] for x in response["portal_types"]]
        self.assertIn("Document", types)

        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISearchSchema, prefix="plone")
        settings.types_not_searched = tuple(
            list(settings.types_not_searched) + ["Document"]
        )
        commit()

        response = self.api_session.get("/@search-filters").json()

        self.assertIn("portal_types", response)
        types = [x["id"] for x in response["portal_types"]]
        self.assertNotIn("Document", types)

    def test_endpoint_do_not_return_contents_that_are_not_searchable(self):
        api.content.create(
            container=self.portal["amministrazione"],
            type="UnitaOrganizzativa",
            title="UO Foo",
            id="uo-foo",
        )
        commit()

        response = self.api_session.get("/@search-filters").json()
        amministrazione = response["sections"][0]["items"][0]

        self.assertEqual("Amministrazione", amministrazione["title"])
        self.assertEqual(8, len(amministrazione["items"]))
        self.assertEqual("UO Foo", amministrazione["items"][-1]["title"])

        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISearchSchema, prefix="plone")
        settings.types_not_searched = tuple(
            list(settings.types_not_searched) + ["UnitaOrganizzativa"]
        )
        commit()

        response = self.api_session.get("/@search-filters").json()
        amministrazione = response["sections"][0]["items"][0]

        self.assertEqual("Amministrazione", amministrazione["title"])
        self.assertEqual(7, len(amministrazione["items"]))

    def test_not_expand_items(self):
        # first section has 7 children
        response = self.api_session.get("/@search-filters").json()
        self.assertEqual(len(response["sections"][0]["items"][0]["items"]), 7)
        self.assertEqual(len(response["sections"][0]["items"][1]["items"]), 15)

        # change expandItems to False for the first section
        settings = json.loads(
            api.portal.get_registry_record(
                "search_sections", interface=IDesignPloneSettings
            )
        )
        settings[0]["items"][0]["expandItems"] = False
        api.portal.set_registry_record(
            "search_sections", json.dumps(settings), interface=IDesignPloneSettings
        )
        commit()

        response = self.api_session.get("/@search-filters").json()
        # first item now has only 1 child
        self.assertEqual(len(response["sections"][0]["items"][0]["items"]), 1)
        # second item still has 15 children
        self.assertEqual(len(response["sections"][0]["items"][1]["items"]), 15)



# -*- coding: utf-8 -*-
from plone import api
from plone.restapi.services import Service
from zope.interface import implementer
from zope.publisher.interfaces import IPublishTraverse


@implementer(IPublishTraverse)
class BandiSearchFiltersGet(Service):
    def reply(self):
        """
        Return possible values based also on current user permissions
        """
        subjects = []
        pc = api.portal.get_tool(name="portal_catalog")
        for subject in pc.uniqueValuesFor("Subject_bando"):
            res = api.content.find(Subject_bando=subject)
            if res:
                subjects.append({"UID": subject, "title": subject})

        office_uids = pc.uniqueValuesFor("ufficio_responsabile_bando")
        offices = [{"UID": x.UID, "title": x.Title} for x in pc(UID=office_uids)]

        subjects.sort(key=lambda x: x["title"])
        offices.sort(key=lambda x: x["title"])
        return {
            "subjects": subjects,
            "offices": offices,
        }



# scrubbing sensitive information
import hashlib
import os
import re
import six


try:
    from collective.sentry import error_handler
except ImportError:
    error_handler = None


SCRUBBING_SENTRY_DATA = (
    os.environ.get("SCRUBBING_SENTRY_DATA", "false").lower() == "true"
)


def hashdata(w):
    return hashlib.md5(w.encode(errors="ignore")).hexdigest()[:9]


def scrub_cookies(cookies):
    for cookie_name in cookies:
        if cookie_name in ("__ac", "auth_token"):
            cookies[cookie_name] = hashdata(cookies[cookie_name])
    return cookies


def scrub_headers(headers):
    for header_name in headers:
        if header_name in ("REMOTE_ADDR", "REMOTE_HOST"):
            headers[header_name] = hashdata(headers[header_name])
    return headers


def scrub_user(user):
    for attr in user:
        if isinstance(user[attr], (six.string_types, six.text_type)):
            user[attr] = hashdata(user[attr])
    return user


def scrub_breadcrumb(breadcrumb):
    if breadcrumb.get("message"):
        breadcrumb["message"] = re.sub(
            r"^(\d+\.\d+\.\d+\.\d+) - ([^ ]+)",
            lambda m: "{} - {}".format(hashdata(m.group(1)), hashdata(m.group(2))),
            breadcrumb["message"],
        )
    return breadcrumb


def scrub_breadcrumbs(values):
    return [scrub_breadcrumb(item) for item in values]


def apply():
    if SCRUBBING_SENTRY_DATA and error_handler:

        def scrubbing(fun):
            def wrapper(event, hint):
                data = fun(event, hint)
                if (data.get("extra") or {}).get("cookies"):
                    data["extra"]["cookies"] = scrub_cookies(data["extra"]["cookies"])
                if ((data.get("extra") or {}).get("request") or {}).get("headers"):
                    data["extra"]["request"]["headers"] = scrub_headers(
                        data["extra"]["request"]["headers"]
                    )
                if data.get("user"):
                    data["user"] = scrub_user(data["user"])
                if (data.get("breadcrumbs") or {}).get("values"):
                    data["breadcrumbs"]["values"] = scrub_breadcrumbs(
                        data["breadcrumbs"]["values"]
                    )
                return data

            return wrapper

        error_handler._before_send = scrubbing(error_handler._before_send)
